<?php

/**
 * This is the model class for table "news".
 *
 * The followings are the available columns in table 'news':
 * @property integer $id
 * @property integer $news_category_id
 * @property string $link
 * @property string $date
 * @property integer $views
 * @property integer $is_silk_way
 * @property string $contacts_data
 * @property integer $gallery_id
 * @property integer $hidden
 * @property string $code
 *
 * The followings are the available model relations:
 * @property NewsCategory $newsCategory
 * @property NewsI18n[] $i18ns
 * @property NewsI18n $i18n
 *
 * @property ImageARBehavior $imageBehavior
 * @property GalleryBehavior  $galleryBehavior
 */
class News extends CActiveRecord implements INodeCrudModel
{
    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return News the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'news';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('news_category_id', 'required'),
            array('news_category_id, views, is_silk_way, gallery_id', 'numerical', 'integerOnly' => true),
            array('date, dateRu, contactIds', 'safe'),
            array('link', 'length', 'max' => 64),
            array('link', 'unique',),
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('id, news_category_id, date, views, is_silk_way, contacts_data', 'safe', 'on' => 'search'),
            array('hidden, code', 'safe'),
        );
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
            'newsCategory' => array(self::BELONGS_TO, 'NewsCategory', 'news_category_id'),
            'i18ns' => array(self::HAS_MANY, 'NewsI18n', 'id', 'index' => 'lang'),
            'i18n' => array(self::HAS_ONE, 'NewsI18n', 'id', 'condition' => 'lang=\'' . Yii::app()->language . '\''),
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => 'ID',
            'news_category_id' => 'Рубрика',
            'date' => 'Дата',
            'dateRu' => 'Дата',
            'views' => 'Количество просмотров',
            'is_silk_way' => 'Относиться у шелковому пути',
            'link' => 'Псевдоним в адресной строке',
            'contactIds' => 'Контакты для отображения в сайдбаре',
            'image' => 'Картинка новости',
            'hidden' => 'Страница скрыта',
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search()
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.

        $criteria = new CDbCriteria;

//////        $criteria->compare('id', $this->id);
        $criteria->compare('news_category_id', $this->news_category_id);
        $criteria->compare('date', $this->date, true);
        $criteria->compare('views', $this->views);
        $criteria->compare('is_silk_way', $this->is_silk_way);
        $criteria->compare('contacts_data', $this->contacts_data, true);

        $sort = new CSort();
        $sort->defaultOrder = array(
            'date' => true,
        );

        return new CActiveDataProvider($this, array(
            'criteria' => $criteria,
            'sort' => $sort,
        ));
    }

    public function save($runValidation = true, $attributes = null)
    {
        $r = parent::save($runValidation, $attributes);
        if (empty($this->link)) {
            $this->link = $this->id;
            $this->saveAttributes($this->attributes);
        }
        return $r;
    }

    public function getContactIds()
    {
        return explode(',', $this->contacts_data);
    }

    public function setContactIds($value)
    {
        if (is_array($value))
            $this->contacts_data = implode(',', $value);
        else
            $this->contacts_data = $value;
    }

    public function getDateRu()
    {
        return ($this->date !== null)
            ? date('d.m.Y H:i:s', CDateTimeParser::parse($this->date, 'yyyy-MM-dd hh:mm:ss'))
            : date('d.m.Y H:i:s');
    }

    public function setDateRu($value)
    {
        $this->date = date('Y-m-d H:i:s', CDateTimeParser::parse($value, 'dd.MM.yyyy hh:mm:ss'));
    }


    public $image;
    public $news_root_id;

    public function behaviors()
    {
        return array(
            'imageBehavior' => array(
                'class' => 'ImageARBehavior',
                'attribute' => 'image', // this must exist
                'extension' => 'png, gif, jpg, jpeg', // possible extensions, comma separated
                'prefix' => 'img_',
                'relativeWebRootFolder' => 'images/news', // this folder must exist

//                'forceExt' => 'png',
                'formats' => array(
                    'preview' => array(
                        'suffix' => '_medium',
                        'process' => array(
                            'centeredpreview' => array(140, 104),
                        ),
                    ),
                    'normal' => array(),
                ),

                'defaultName' => 'default',
            ),
            'galleryBehavior' => array(
                'class' => 'GalleryBehavior',
                'idAttribute' => 'gallery_id',
                'versions' => array(
                    'small' => array(
                        'centeredpreview' => array(140, 105),
                    ),
                    'medium' => array(
                        'cresize' => array(800, null),
                        'watermark' => array('images/watermark.png', 0, 0),
                    )
                ),
                'name' => true,
                'description' => true,
            ),
        );
    }

    public function getCategories()
    {

        $criteria = new CDbCriteria();
        $criteria->addCondition('type=1 or type=3');
        $criteria->compare('news_root_id', $this->news_root_id);

        return CHtml::listData(NewsCategory::model()->with('i18n')->findAll($criteria), 'id', 'i18n.name');
    }

    /**
     * @param Node $node
     * @return void
     */
    public function setNode($node)
    {
        $this->news_root_id = $node->data_id;
    }
}