<?php

/**
 * This is the model class for table "news_category".
 *
 * The followings are the available columns in table 'news_category':
 * @property integer $id
 * @property integer $rank
 * @property integer $type
 * @property string $contacts_data
 * @property string $link
 * @property integer $gallery_id
 * @property integer $photofact_gallery_id
 * @property integer $news_root_id
 * @property integer $hidden
 * @property string $code
 *
 * The followings are the available model relations:
 * @property News[] $news
 * @property NewsCategoryI18n[] $i18ns
 * @property NewsCategoryI18n $i18n
 * @property Publication[] $publications
 *
 * @property GalleryBehavior  $galleryBehavior
 * @property GalleryBehavior  $photofactBehavior
 */
class NewsCategory extends CActiveRecord  implements INodeCrudModel
{
    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return NewsCategory the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'news_category';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('rank, type', 'numerical', 'integerOnly' => true),
            array('contactIds', 'safe'),
            array('link', 'length', 'max' => 64),
            array('link', 'unique',),
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('id, rank, type, contacts_data', 'safe', 'on' => 'search'),
            array('hidden, code', 'safe'),
        );
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
            'news' => array(self::HAS_MANY, 'News', 'news_category_id'),
            'i18ns' => array(self::HAS_MANY, 'NewsCategoryI18n', 'id', 'index' => 'lang'),
            'i18n' => array(self::HAS_ONE, 'NewsCategoryI18n', 'id', 'condition' => 'lang=\'' . Yii::app()->language . '\''),
            'publications' => array(self::HAS_MANY, 'Publication', 'news_category_id'),
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => 'ID',
            'rank' => 'Rank',
            'type' => 'Тип',
            'link' => 'Псевдоним в адресной строке',
            'contactIds' => 'Контакты для отображения в сайдбаре',
            'hidden' => 'Страница скрыта',
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search()
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.

        $criteria = new CDbCriteria;

////        $criteria->compare('id', $this->id);
        $criteria->compare('news_root_id', $this->news_root_id);
        $criteria->compare('rank', $this->rank);
        $criteria->compare('type', $this->type);
        $criteria->compare('contacts_data', $this->contacts_data, true);

        $sort = new CSort();
        $sort->defaultOrder = array(
            'rank' => false,
        );

        return new CActiveDataProvider($this, array(
            'criteria' => $criteria,
            'sort' => $sort,
        ));
    }

    public function save($runValidation = true, $attributes = null)
    {
        $r = parent::save($runValidation, $attributes);
        if (empty($this->link)) {
            $this->link = $this->id;
            $this->saveAttributes($this->attributes);
        }
        return $r;
    }

    public function getContactIds()
    {
        return explode(',', $this->contacts_data);
    }

    public function setContactIds($value)
    {
        if (is_array($value))
            $this->contacts_data = implode(',', $value);
        else
            $this->contacts_data = $value;
    }

    public function getTypesData()
    {
        return array(
            1 => 'Новости',
            2 => 'Публикации',
            3 => 'Фотофакты',
            4 => 'Нас показывают',
        );
    }

    public function behaviors()
    {
        return array(
            'galleryBehavior' => array(
                'class' => 'GalleryBehavior',
                'idAttribute' => 'gallery_id',
                'versions' => array(
                    'gallery' => array(
                        'centeredpreview' => array(1008, 115),
                    )
                ),
                'name' => true,
                'description' => true,
            ),
            'photofactBehavior' => array(
                'class' => 'GalleryBehavior',
                'idAttribute' => 'photofact_gallery_id',
                'versions' => array(
                    'small' => array(
                        'centeredpreview' => array(140, 104),
                    )
                ),
                'name' => true,
                'description' => false,
                'link'=>true,
            ),
        );
    }

    /**
     * @param Node $node
     * @return void
     */
    public function setNode($node)
    {
        $this->news_root_id = $node->data_id;
    }
}