<?php

/**
 * This is the model class for table "partner".
 *
 * The followings are the available columns in table 'partner':
 * @property integer $id
 * @property string $contacts_data
 * @property string $link
 * @property string $rank
 * @property string $gallery_id
 * @property string $header_gallery_id
 * @property integer $partners_section_id
 * @property integer $hidden
 * @property string $code
 *
 * The followings are the available model relations:
 * @property Brand[] $brands
 * @property PartnerI18n[] $i18ns
 * @property PartnerI18n $i18n
 *
 * @property ImageARBehavior $imageBehavior
 * @property GalleryBehavior  $galleryBehavior
 * @property GalleryBehavior  $headerGalleryBehavior
 */
class Partner extends CActiveRecord implements INodeCrudModel
{
    public $image;
    public $_brandIds;

    /**
     * Returns the static model of the specified AR class.
     * @param string $className active record class name.
     * @return Partner the static model class
     */
    public static function model($className = __CLASS__)
    {
        return parent::model($className);
    }

    /**
     * @return string the associated database table name
     */
    public function tableName()
    {
        return 'partner';
    }

    /**
     * @return array validation rules for model attributes.
     */
    public function rules()
    {
        // NOTE: you should only define rules for those attributes that
        // will receive user inputs.
        return array(
            array('link', 'length', 'max' => 64),
            array('link', 'unique',),
            array('contactIds, brandIds, rank, gallery_id', 'safe'),
            // The following rule is used by search().
            // Please remove those attributes that should not be searched.
            array('id, contacts_data, link', 'safe', 'on' => 'search'),
            array('hidden, code', 'safe'),
        );
    }

    /**
     * @return array relational rules.
     */
    public function relations()
    {
        // NOTE: you may need to adjust the relation name and the related
        // class name for the relations automatically generated below.
        return array(
            'brands' => array(self::MANY_MANY, 'Brand', 'partner_has_brand(partner_id, brand_id)', 'index' => 'id'),
            'i18ns' => array(self::HAS_MANY, 'PartnerI18n', 'id', 'index' => 'lang'),
            'i18n' => array(self::HAS_ONE, 'PartnerI18n', 'id', 'condition' => 'lang=\'' . Yii::app()->language . '\''),
        );
    }

    /**
     * @return array customized attribute labels (name=>label)
     */
    public function attributeLabels()
    {
        return array(
            'id' => 'ID',
            'link' => 'Псевдоним в адресной строке',
            'contactIds' => 'Контакты для отображения в сайдбаре',
            'brandIds' => 'Бренды',
            'image' => 'Лого',
            'hidden' => 'Страница скрыта',
        );
    }

    /**
     * Retrieves a list of models based on the current search/filter conditions.
     * @return CActiveDataProvider the data provider that can return the models based on the search/filter conditions.
     */
    public function search()
    {
        // Warning: Please modify the following code to remove attributes that
        // should not be searched.

        $criteria = new CDbCriteria;

//        $criteria->compare('id', $this->id);
        $criteria->compare('partners_section_id', $this->partners_section_id);
        $criteria->compare('contacts_data', $this->contacts_data, true);
        $criteria->compare('link', $this->link, true);

        $sort = new CSort();
        $sort->defaultOrder = array(
            'rank' => false,
        );

        return new CActiveDataProvider($this, array(
            'criteria' => $criteria,
            'sort' => $sort,
        ));
    }


    public function getContactIds()
    {
        return explode(',', $this->contacts_data);
    }

    public function setContactIds($value)
    {
        if (is_array($value))
            $this->contacts_data = implode(',', $value);
        else
            $this->contacts_data = $value;
    }

    public function save($runValidation = true, $attributes = null)
    {
        $r = parent::save($runValidation, $attributes);
        if (empty($this->link)) {
            $this->link = $this->id;
            $this->saveAttributes($this->attributes);
        }
        return $r;
    }

    public function behaviors()
    {
        return array(
            'imageBehavior' => array(
                'class' => 'ImageARBehavior',
                'attribute' => 'image', // this must exist
                'extension' => 'png, gif, jpg, jpeg', // possible extensions, comma separated
                'prefix' => 'img_',
                'relativeWebRootFolder' => 'images/partners', // this folder must exist

                'forceExt' => 'png',
                'formats' => array(
                    'medium' => array(
                        'suffix' => '_medium',
                        'process' => array(
                            'centeredpreview' => array(194, 145),
//                            'crop' => array(133, 57, 'center'),
                        ),
                    ),
                    'normal' => array(),
                ),

                'defaultName' => 'default',
            ),
            'galleryBehavior' => array(
                'class' => 'GalleryBehavior',
                'idAttribute' => 'gallery_id',
                'versions' => array(
                    'small' => array(
                        'centeredpreview' => array(250, 187),
                    ),
                    'medium' => array(
                        'cresize' => array(800, null),
                        'watermark'=>array('images/watermark.png',0,0)  ,
                    )
                ),
                'name' => true,
                'description' => true,
            ),
            'headerGalleryBehavior' => array(
                'class' => 'GalleryBehavior',
                'idAttribute' => 'header_gallery_id',
                'versions' => array(
                    'gallery' => array(
                        'centeredpreview' => array(1008, 115),
                    )
                ),
                'name' => true,
                'description' => true,
            ),
        );
    }


    public function getBrandIds()
    {
        if (empty($this->_brandIds)) {
            $res = array();
            foreach ($this->brands as $brand) {
                $res[] = $brand->id;
            }
            $this->_brandIds = $res;
        }
        return $this->_brandIds;
    }

    public function setBrandIds($value)
    {
        $this->_brandIds = $value;
    }

    protected function afterSave()
    {
        parent::afterSave();

        $brandIds = $this->getBrandIds();
        foreach ($this->brands as $brand) {
            if (!in_array($brand->id, $brandIds)) {
                PartnerHasBrand::model()->deleteByPk(array('partner_id' => $this->id, 'brand_id' => $brand->id));
            }
        }
        foreach ($brandIds as $id) {
            if (!isset($this->brands[$id])) {
                $rel = new PartnerHasBrand();
                $rel->partner_id = $this->id;
                $rel->brand_id = $id;
                $rel->save();
            }
        }
    }

    /**
     * @param Node $node
     * @return void
     */
    public function setNode($node)
    {
        $this->partners_section_id = $node->data_id;
    }

}